;==========================================================================================================================================================
; Main includes
;==========================================================================================================================================================

XIncludeFile "NotePiddle_Compiler.pb"                                                               ; Compiler check constants for RichEdit handling
XIncludeFile "NotePiddle_Constants.pb"                                                              ; Visual designer created constantscode
XIncludeFile "NotePiddle_Windows.pb"                                                                ; Visual designer created windows code
XIncludeFile "NotePiddle_MyDeclarations.pb"                                                         ; My procedural declarations
XIncludeFile "NotePiddle_MyConstants.pb"                                                            ; My own constants list

;==========================================================================================================================================================
; Third party stuff I don't understand but works
;==========================================================================================================================================================

XIncludeFile "Modules\ThirdParty\_MenuColourPicker.pbi"                                             ; SROD. Show a POP-up menu which contains a selection of 'standard colour' cells
XIncludeFile "Modules\ThirdParty\_OwnerdrawComboIconsFonts.pbi"                                     ; FluidByte code to retrieve the currently installed windows fonts

;==========================================================================================================================================================
; Freak, Srod, Sparkie, Richedit handling
;==========================================================================================================================================================

XIncludeFile "Modules\RTF Handling\_Editor_Bulleted.pbi"                                            ; Set bulleted text mode
XIncludeFile "Modules\RTF Handling\_Editor_Color.pbi"                                               ; Set font colour
XIncludeFile "Modules\RTF Handling\_Editor_FindTextn.pbi"                                           ; Find whole text strings with no splitting
XIncludeFile "Modules\RTF Handling\_Editor_Font.pbi"                                                ; Set font type
XIncludeFile "Modules\RTF Handling\_Editor_FontSize.pbi"                                            ; Set font size
XIncludeFile "Modules\RTF Handling\_Editor_Format.pbi"                                              ; Set font style (Bold, italic etc)
XIncludeFile "Modules\RTF Handling\_Editor_JustifyParagraph.pbi"                                    ; Set left, center, right justify
XIncludeFile "Modules\RTF Handling\_Editor_Load.pbi"                                                ; Load an RTF file
XIncludeFile "Modules\RTF Handling\_Editor_Save.pbi"                                                ; Save an RTF file
XIncludeFile "Modules\RTF Handling\_Editor_StreamCallback.pbi"                                      ; RTF load and save stream callback
XIncludeFile "Modules\RTF Handling\_Editor_Wrap.pbi"                                                ; Turn editor word wrap on

;==========================================================================================================================================================
; Me, wrapper routines for the RTF stuff
;==========================================================================================================================================================

XIncludeFile "Modules\RTF Wrappers\_Run_Editor_FindTextn.pbi"                                       ; Search for text in the document
XIncludeFile "Modules\RTF Wrappers\_Run_Editor_Save.pbi"                                            ; Wrapper for "Save an RTF file"
XIncludeFile "Modules\RTF Wrappers\_Run_Editor_Load.pbi"                                            ; Wrapper for "Load an RTF file"
XIncludeFile "Modules\RTF Wrappers\_Run_Editor_Color.pbi"                                           ; Wrapper for "Set font colour" and "MenuColourPicker"
XIncludeFile "Modules\RTF Wrappers\_Run_Editor_Font.pbi"                                            ; Wrapper for Editor_Font
XIncludeFile "Modules\RTF Wrappers\_Run_Editor_FontSize.pbi"                                        ; Resize font with Control + Up and Down keys

;==========================================================================================================================================================
; Any generic routines
;==========================================================================================================================================================

XIncludeFile "Modules\Generic\_CloseMyWindow.pbi"                                                   ; Fade the window out gently, close it and reset the mutex
XIncludeFile "Modules\Generic\_FadeInMyWindow.pbi"                                                  ; Fade a form window into the display
XIncludeFile "Modules\Generic\_RemoveBorders.pbi"                                                   ; Remove borders from various gadgets
XIncludeFile "Modules\Generic\_SetStatusBar.pbi"                                                    ; Handle statusbar messages with less typing.
XIncludeFile "Modules\Generic\_SetStyle.pbi"                                                        ; Set the window style for various effects

;==========================================================================================================================================================
; EditorGadget details such as lines, words and characters
;==========================================================================================================================================================

XIncludeFile "Modules\Editor\_GetEditorDetails.pbi"                                                 ; Get all details like line, char, count etc

;==========================================================================================================================================================
; Open the text search window
;==========================================================================================================================================================

XIncludeFile "Modules\Windows\_OpenSearchWindow.pbi"                                                ; Open the text search window
XIncludeFile "Modules\Windows\_HideProgramWindow.pbi"                                               ; Hide the program window
XIncludeFile "Modules\Windows\_ShowProgramWindow.pbi"                                               ; Show the program window

;==========================================================================================================================================================
; Main program code
;==========================================================================================================================================================

If Window_notepiddle()

  ;------------------------------------------------------------------------------------------------
  ; Set the initial program quit value to zero
  ;------------------------------------------------------------------------------------------------
  Program\QuitValue = 0
  
  ;------------------------------------------------------------------------------------------------
  ; Turn form fading on
  ;------------------------------------------------------------------------------------------------
  Program\Fade = 1

  ;------------------------------------------------------------------------------------------------
  ; Turn snap forms to borders on
  ;------------------------------------------------------------------------------------------------
  Program\ScreenSnap = 1

  ;------------------------------------------------------------------------------------------------
  ; Allow the editorgadget to send back event messages. Sparkie/Srod
  ;------------------------------------------------------------------------------------------------
  CurrentFlags.i = SendMessage_(GadgetID(#Gadget_notepiddle_edit), #EM_GETEVENTMASK, 0, 0)
  SendMessage_(GadgetID(#Gadget_notepiddle_edit), #EM_SETEVENTMASK, 0, #ENM_MOUSEEVENTS | #ENM_CHANGE | CurrentFlags.i) 

  ;------------------------------------------------------------------------------------------------
  ; FluidByte routine. Set ownerdrawn combo box's item height
  ;------------------------------------------------------------------------------------------------
  SendMessage_(GadgetID(#Gadget_notepiddle_font), #CB_SETITEMHEIGHT, 0, 20)

  ;------------------------------------------------------------------------------------------------
  ; FluidByte routine. Find all the fonts in the system
  ;------------------------------------------------------------------------------------------------
  EnumFonts(#Gadget_notepiddle_font)
  
  ;------------------------------------------------------------------------------------------------
  ; FluidByte routine. Make sure the font selector is highlighted
  ;------------------------------------------------------------------------------------------------
  SetActiveGadget(#Gadget_notepiddle_font)

  ;------------------------------------------------------------------------------------------------
  ; FluidByte routine. Set the highlight to the first font in the list
  ;------------------------------------------------------------------------------------------------
  SetGadgetState(#Gadget_notepiddle_font, 0)
  
  ;------------------------------------------------------------------------------------------------
  ; Remove theme support for certain gadgets where colours don't seem to work in PureBasic. This is
  ; as there is no crossplatform colour support for these controls
  ;------------------------------------------------------------------------------------------------
  SetWindowTheme_(GadgetID(#Gadget_notepiddle_edit),          @null.w, @null.w)
  
  ;------------------------------------------------------------------------------------------------
  ; Remove the borders from some gadgets that don't have pb comands to do so. This makes the 'flat'
  ; look more consistent.
  ;------------------------------------------------------------------------------------------------
  RemoveBorders(#Gadget_notepiddle_edit)
  RemoveBorders(#Gadget_notepiddle_font)
  
  ;------------------------------------------------------------------------------------------------
  ; Setup the windows messaging queue handler. FluidByte and NetMaestro stuff mainly
  ;------------------------------------------------------------------------------------------------
  SetWindowCallback(@WindowCallback())
  
  ;------------------------------------------------------------------------------------------------
  ; Add keyboard shortcuts as backup to the buttons, Good for people with sore mouse wrists
  ;------------------------------------------------------------------------------------------------
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_B,    #Shortcut_Bold)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_U,    #Shortcut_Underline)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_I,    #Shortcut_Italic)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_S,    #Shortcut_Strikeout)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_A,    #Shortcut_Subscript)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_D,    #Shortcut_Superscript)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_L,    #Shortcut_Link)

  ;------------------------------------------------------------------------------------------------
  ; Make the selected font grow or shrink
  ;------------------------------------------------------------------------------------------------
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_Up,   #Shortcut_FontUp)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_Down, #Shortcut_FontDown)
  
  ;------------------------------------------------------------------------------------------------
  ; Fast exit from the program
  ;------------------------------------------------------------------------------------------------
 
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Alt     | #PB_Shortcut_L,    #Shortcut_Load)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Alt     | #PB_Shortcut_S,    #Shortcut_Save)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Alt     | #PB_Shortcut_E,    #Shortcut_Exit)
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Control | #PB_Shortcut_F,    #Shortcut_Search)
  
  ;------------------------------------------------------------------------------------------------
  ; Bring up program help
  ;------------------------------------------------------------------------------------------------
  AddKeyboardShortcut(#Window_notepiddle, #PB_Shortcut_Alt     | #PB_Shortcut_H,    #Shortcut_Help)
  
  ;------------------------------------------------------------------------------------------------
  ; Remove the 32k character limit in the richedit gadget
  ;------------------------------------------------------------------------------------------------
  SendMessage_(GadgetID(#Gadget_notepiddle_edit), #EM_LIMITTEXT, -1, 0)

  ;------------------------------------------------------------------------------------------------
  ; Make the edit box receive focus
  ;------------------------------------------------------------------------------------------------
  SetActiveGadget(#Gadget_notepiddle_edit)

  ;------------------------------------------------------------------------------------------------
  ; Add the standard font sizes to the size selector (Could be added to a data section)
  ;------------------------------------------------------------------------------------------------
  Restore BeginFontSizes
  While TempFontsizes.s <> "_EndFontSizes_"
    Read.s TempFontsizes.s
    If TempFontsizes.s <> "_EndFontSizes_"
      AddGadgetItem(#Gadget_notepiddle_size, -1, TempFontsizes.s)
    EndIf
  Wend
  TempFontsize.s    = "<>"
  
  SetGadgetState(#Gadget_notepiddle_size, 0)
      
  ;--------------------------------------------------------------------------------------------
  ; Add the standard regional character sets to the selector (Not used yet)
  ;--------------------------------------------------------------------------------------------
  
  Restore BeginCharacterArea
  While TempCharacterArea.s <> "_EndCharacterArea_"
    Read.s TempCharacterArea.s
    If TempCharacterArea.s <> "_EndCharacterArea_"
      AddGadgetItem(#Gadget_notepiddle_character, -1, TempCharacterArea.s)
    EndIf
  Wend
  TempFontsize.s    = "<>"
      
  SetGadgetState(#Gadget_notepiddle_character, 8)
  
  ;------------------------------------------------------------------------------------------------
  ; Find character defaults for the editor
  ;------------------------------------------------------------------------------------------------
  SendMessage_(GadgetID(#Gadget_notepiddle_edit), #EM_GETCHARFORMAT, #SC_DEFAULT, defaultFormat)
  defaultFormat\cbSize = SizeOf(CHARFORMAT)
  defaultFormat\dwMask = #CFM_ITALIC  | #CFM_BOLD | #CFM_STRIKEOUT | #CFM_UNDERLINE

  ;------------------------------------------------------------------------------------------------
  ; Put in a starting message in the statusbar
  ;------------------------------------------------------------------------------------------------
  SetStatusBar(1, "NotePiddle, v" + #TrayNote)
  SetStatusBar(3, "0")
    
  ;------------------------------------------------------------------------------------------------
  ; Fade in the window.
  ;------------------------------------------------------------------------------------------------
  FadeInMyWindow(#Window_notepiddle, 10)
  
  ;------------------------------------------------------------------------------------------------
  ; Main program event handler
  ;------------------------------------------------------------------------------------------------
  
  Repeat
    Select WaitWindowEvent()
      Case #PB_Event_CloseWindow
        ;------------------------------------
        ; 
        ;------------------------------------
        Select EventWindow()
          Case #Window_notepiddle           : Program\QuitValue = 1
          Case #Window_Search               : CloseMyWindow(#Window_Search, 8)
        EndSelect
        ;------------------------------------
        ; Minimise the program to the windows tray and set icon and tooltip
        ;------------------------------------
      Case #PB_Event_MinimizeWindow
        Select EventWindow()
          Case #Window_NotePiddle           : HideProgramWindow()
        EndSelect
        ;------------------------------------
        ; 
        ;------------------------------------
      Case #PB_Event_Menu
        Select EventMenu()
          ;----------------------------------
          ; Main form menubar
          ;----------------------------------
          Case #MenuBar_load                : Run_Editor_Load(#Gadget_notepiddle_edit)
          Case #MenuBar_save                : Run_Editor_Save(#Gadget_notepiddle_edit)
          Case #MenuBar_exit                : Program\QuitValue = 1
          Case #MenuBar_Search              : OpenSearchWindow()
          Case #MenuBar_help                : 
          ;----------------------------------
          ; Main form text effects
          ;----------------------------------
          Case #Shortcut_Bold               : Editor_Format(#Gadget_notepiddle_edit, #CFE_BOLD,        1)
          Case #Shortcut_Italic             : Editor_Format(#Gadget_notepiddle_edit, #CFE_ITALIC,      1)
          Case #Shortcut_Underline          : Editor_Format(#Gadget_notepiddle_edit, #CFE_UNDERLINE,   1)
          Case #Shortcut_Strikeout          : Editor_Format(#Gadget_notepiddle_edit, #CFE_STRIKEOUT,   1)
          Case #Shortcut_Superscript        : Editor_Format(#Gadget_notepiddle_edit, #CFE_SUPERSCRIPT, 1)
          Case #Shortcut_Subscript          : Editor_Format(#Gadget_notepiddle_edit, #CFE_SUBSCRIPT,   1)
          Case #Shortcut_Link               : Editor_Format(#Gadget_notepiddle_edit, #CFE_LINK,        1)
          Case #Shortcut_FontUp             : Run_Editor_FontSize(#Gadget_notepiddle_edit, "Up")
          Case #Shortcut_FontDown           : Run_Editor_FontSize(#Gadget_notepiddle_edit, "Down")
          ;----------------------------------
          ; Main form menubar shortcuts
          ;----------------------------------
          Case #Shortcut_Load               : Run_Editor_Load(#Gadget_notepiddle_edit)
          Case #Shortcut_Save               : Run_Editor_Save(#Gadget_notepiddle_edit)
          Case #Shortcut_Exit               : Program\QuitValue = 1
          Case #Shortcut_Search             : OpenSearchWindow()
          Case #Shortcut_Help               : 
          ;----------------------------------
          ; Search form controls
          ;----------------------------------
          Case #Shortcut_Search_SearchFirst   ; Remove shortcut to make return work as a find first action, F3 is find next
                                              RemoveKeyboardShortcut(#Window_Search, #PB_Shortcut_Return)
                                              Run_Editor_FindTextn(#Gadget_notepiddle_edit) 
          Case #Shortcut_Search_SearchNext  : Run_Editor_FindTextn(#Gadget_notepiddle_edit)
          Case #Shortcut_Search_escape      : CloseMyWindow(#Window_Search, 8)
          ;----------------------------------
          ; 
          ;----------------------------------
        EndSelect
        ;------------------------------------
        ; 
        ;------------------------------------
      Case #PB_Event_Gadget
        Select EventGadget()
          ;----------------------------------
          ; 
          ;----------------------------------
          Case #Gadget_notepiddle_font
            Select EventType()
              Case #PB_EventType_Change     : Run_Editor_Font(#Gadget_notepiddle_edit) ; Wrapper for Editor_Font
            EndSelect
            ;--------------------------------
            ; 
            ;--------------------------------
          Case #Gadget_notepiddle_size
            Select EventType()
              Case #PB_EventType_Change     : Editor_FontSize(#Gadget_notepiddle_edit, Val(GetGadgetText(#Gadget_notepiddle_size)))
            EndSelect
            ;--------------------------------
            ; 
            ;--------------------------------
          Case #Gadget_notepiddle_character : ; SetSelectedCharacterLocale()
          Case #Gadget_notepiddle_bold      : Editor_Format(#Gadget_notepiddle_edit, #CFE_BOLD,        1)
          Case #Gadget_notepiddle_italic    : Editor_Format(#Gadget_notepiddle_edit, #CFE_ITALIC,      1)
          Case #Gadget_notepiddle_underline : Editor_Format(#Gadget_notepiddle_edit, #CFE_UNDERLINE,   1)
          Case #Gadget_notepiddle_colour    : Run_Editor_Color(#Gadget_notepiddle_edit)
          Case #Gadget_notepiddle_left      : Editor_JustifyParagraph(#Gadget_notepiddle_edit, #PFA_LEFT)
          Case #Gadget_notepiddle_middle    : Editor_JustifyParagraph(#Gadget_notepiddle_edit, #PFA_CENTER)
          Case #Gadget_notepiddle_right     : Editor_JustifyParagraph(#Gadget_notepiddle_edit, #PFA_RIGHT)
          Case #Gadget_notepiddle_bullet    : Editor_Bulleted(#Gadget_notepiddle_edit)
          ;----------------------------------
          ; 
          ;----------------------------------
          Case #Gadget_notepiddle_edit
            Select EventType()
              Case #PB_EventType_Change     : GetEditorDetails()
            EndSelect
            ;--------------------------------
            ; Search form cue text control
            ;--------------------------------
          Case #Gadget_Search_Search
            If GetGadgetText(#Gadget_Search_Search) = ""
              SetGadgetFont(#Gadget_Search_Search, FontID(Program\SearchCueFont))
            Else
              SetGadgetFont(#Gadget_Search_Search, FontID(Program\SearchNormalFont))
            EndIf
            ;----------------------------------
            ; 
            ;----------------------------------
          Case #Gadget_Search_SearchUp      : Run_Editor_FontSize(#Gadget_notepiddle_edit, "Up")
          Case #Gadget_Search_SearchDown    : Run_Editor_FontSize(#Gadget_notepiddle_edit, "Down")
          Case #Gadget_Search_Searchnext    : Run_Editor_FindTextn(#Gadget_notepiddle_edit)
          Case #Gadget_Search_cancel        : CloseMyWindow(#Window_Search, 8)
          ;----------------------------------
          ; 
          ;----------------------------------
        EndSelect
        ;------------------------------------------------------------------------------------------
        ; Unhide the program window when the user clicks the icon in the tray
        ;------------------------------------------------------------------------------------------
      Case #PB_Event_SysTray                          : ShowProgramWindow()
    EndSelect
  Until Program\QuitValue
  ;------------------------------------------------------------------------------------------------
  ; FluidByte. Destroy the imagelist created for the owner drawn combo box
  ;------------------------------------------------------------------------------------------------
  ImageList_Destroy_(himlFontType)
  ;------------------------------------------------------------------------------------------------
  ; Close the window, fading it gently off the screen
  ;------------------------------------------------------------------------------------------------
  CloseMyWindow(#Window_notepiddle, 8)
  ;------------------------------------------------------------------------------------------------
  ; End the program
  ;------------------------------------------------------------------------------------------------
EndIf
  ;------------------------------------------------------------------------------------------------
  ; Remove program stub from memory
  ;------------------------------------------------------------------------------------------------
End

; So the fonts show up properly as graphics in the combo box, these flags always need to be added to
; the window code. Since the window code is usually overwritten by a visual designer, I must remind
; myself of them here.
; 
; #CBS_OWNERDRAWFIXED | #CBS_HASSTRINGS
; IDE Options = PureBasic 5.23 LTS (Windows - x86)
; CursorPosition = 89
; FirstLine = 84
; EnableAsm
; EnableXP
; UseIcon = Images\notes16x16.ico
; Executable = ..\..\NotePiddle.exe