;==========================================================================================================================================================
; Main program data structure
;==========================================================================================================================================================

Structure ProgramData
  QuitValue.i                                                                                       ; Program quit semaphore
  FontSize.i                                                                                        ; Currently used font size
  Fade.i                                                                                            ; Fade a form window into the display
  ScreenSnap.i                                                                                      ; Snap forms to screen borders
  TrayState.i                                                                                       ; Is the form minimised to the system tray
  
  SearchCueFont.i                                                                                   ; Searchbox cue font
  SearchNormalFont.i                                                                                ; Searchbox normal font
  SearchCueText.s                                                                                   ; Search box prompt text
  
  StaticBack.i                                                                                      ; CheckBox, Optiongadget background colours with callback
EndStructure

;==========================================================================================================================================================
; Other variables that could be put into the structure above
;==========================================================================================================================================================

Global  Program.ProgramData

Global  NewList FaceName.s()

Global  RTFLength, FontSize
Global  RtfFile
Global  defaultFormat.CHARFORMAT

;==========================================================================================================================================================
; Program version, heading and tray
;==========================================================================================================================================================

#Version  = "0.01"

#Traynote =  #Version + " Bought to you by the letter 'Foo'"                                        ; Copyright string

;==========================================================================================================================================================
; Default startup font size
;==========================================================================================================================================================

Program\FontSize = 14

;==========================================================================================================================================================
; Variables for InfraTec's RTF search procedure
;==========================================================================================================================================================

Define *Buffer, i, ExitProc.i, Event

;==========================================================================================================================================================
; NetMaestro's screen snap code
;==========================================================================================================================================================

Global screenW, screenH, SnapD = 50, AutoSnap = #True

SystemParametersInfo_(#SPI_GETWORKAREA, 0, @wa.RECT, 0)                                             ; Client area of the desktop

ScreenW = wa\right

screenH = wa\bottom

;==========================================================================================================================================================
; Setup the search box cue fonts
;==========================================================================================================================================================

Program\SearchCueFont       = LoadFont(#PB_Any,   "Georgia",       13, #PB_Font_Italic)             ; Empty search cue font
Program\SearchNormalFont    = LoadFont(#PB_Any,   "Comic Sans MS", 13, 0)                           ; Filled cue box font
Program\SearchCueText       = Space(256)                                                            ; Set the cue text buffer

PokeS(@Program\SearchCueText, "Search string here...", -1, #PB_Unicode)                             ; Fill the buffer with the cue text

;==========================================================================================================================================================
; Used in a callback to colour static controls that PB doesn't manage
;==========================================================================================================================================================

Program\StaticBack          = CreateSolidBrush_($BFBFBF)                                            ; Static controls background colours

;==========================================================================================================================================================
; Set initial tray hiding state so we avoing loops
;==========================================================================================================================================================

Program\TrayState     = #False

;==========================================================================================================================================================
; Srod. Colour picker module. Define a color array. This is optional and consists of 12 elements, not all of which have to be defined.
;==========================================================================================================================================================

Global Dim Colours(11)

; In this example we define 4 custom colors and mark the remaining 8 as undefined by setting them to -1. 
; (The user will not be able to select these undefined colours.)

  Colours(0) = #Green
  Colours(1) = #Blue
  Colours(2) = #Gray
  Colours(3) = #Red

For i = 4 To 11
  Colours(i) = -1
Next

;==========================================================================================================================================================
; Default locale character set
;==========================================================================================================================================================

lf.LOGFONT
lf\lfCharset = #DEFAULT_CHARSET

;==========================================================================================================================================================
; Menus, shortcuts and gadgets
;==========================================================================================================================================================

Enumeration #GadgetIndex
  ; Tray area
  #Systray_notepiddle_tray
  
  ; Font shortcuts
  #Shortcut_Bold
  #Shortcut_Underline
  #Shortcut_Italic
  #Shortcut_Strikeout
  #Shortcut_Subscript
  #Shortcut_Superscript
  #Shortcut_Link
  #Shortcut_FontUp
  #Shortcut_FontDown
  
  ; File shortcuts
  #Shortcut_Save
  #Shortcut_Load
  #Shortcut_Exit
  #Shortcut_Search
  #Shortcut_Help
  
  ; Find shortcut
  #Shortcut_Search_SearchFirst
  #Shortcut_Search_SearchNext
  #Shortcut_Search_escape
EndEnumeration

;==========================================================================================================================================================
; All my image constants
;==========================================================================================================================================================

Enumeration #ImageIndex
  #Image_notepiddle_tray                                                                            ; Program tray icon
  #Image_notepiddle_messages                                                                        ; Statusbar message field
EndEnumeration

;==========================================================================================================================================================
; Catch images from memory
;==========================================================================================================================================================

CatchImage(#Image_notepiddle_messages,  ?_MGB_notepiddle_messages)                                  ; messages16x16.ico
CatchImage(#Image_notepiddle_tray,      ?_MGB_notepiddle_tray)                                      ; database16x16.ico

;==========================================================================================================================================================
; Include images in the exe file
;==========================================================================================================================================================

DataSection
  _MGB_notepiddle_messages: : IncludeBinary "Images\messages16x16.ico"
  _MGB_notepiddle_tray:     : IncludeBinary "Images\notes16x16.ico"
  
  BeginFontSizes:
  
  Data.s  "8"
  Data.s  "9"
  Data.s "10"
  Data.s "11"
  Data.s "12"
  Data.s "14"
  Data.s "16"
  Data.s "18"
  Data.s "20"
  Data.s "22"
  Data.s "24"
  Data.s "26"
  Data.s "28"
  Data.s "36"
  Data.s "48"
  Data.s "72"
  Data.s  "_EndFontSizes_"
  
  BeginCharacterArea:
    
  Data.s "Arabic"
  Data.s "Baltic"
  Data.s "Central European"
  Data.s "Cyrillic"
  Data.s "Greek"
  Data.s "Hebrew"
  Data.s "Turkish"
  Data.s "Vietnamese"
  Data.s "Western"
  Data.s  "_EndCharacterArea_"
EndDataSection

; IDE Options = PureBasic 5.23 LTS (Windows - x86)
; CursorPosition = 137
; FirstLine = 127
; EnableAsm
; EnableXP